import 'package:flutter/material.dart';
import 'package:hypi_tink_api/hypi_tink_api.dart';

enum TinkRenderMode { editor, preview, runtime }

extension TinkRenderModeExtension on TinkRenderMode {
  static TinkRenderMode of(String str) {
    switch (str) {
      case 'editor':
        return TinkRenderMode.editor;
      case 'preview':
        return TinkRenderMode.preview;
      case 'runtime':
        return TinkRenderMode.runtime;
    }
    return TinkRenderMode.editor;
  }

  String name() {
    switch (this) {
      case TinkRenderMode.editor:
        return 'editor';
      case TinkRenderMode.preview:
        return 'preview';
      case TinkRenderMode.runtime:
        return 'runtime';
    }
  }
}

mixin ScreenModel {
  String get path;

  Widget render();

  void removeChild(TinkWidgetHandler h);
}

abstract class PropertyModel with Displayable {
  @override
  final String id;
  @override
  final String label;
  @override
  final String groupId;

  PropertyModel(this.id, this.label, this.groupId);
}

abstract class EventModel with Displayable {
  String get name;
}

abstract class WidgetDefModel<P extends PropertyModel, E extends EventModel>
    with Displayable {
  Set<P> properties;
  Set<E> events;
  @override
  final String id;
  @override
  final String label;
  @override
  final String groupId;

  WidgetDefModel(
    this.id,
    this.label,
    this.groupId,
    this.properties,
    this.events,
  );

  create(Map<String, dynamic> args);
}

class TinkParam {
  final String name;
  final String? desc;
  final TinkVariableType type;
  final bool required;

  const TinkParam(
    this.name,
    this.type, {
    this.required = true,
    this.desc,
  });

  @override
  String toString() {
    return name + ':' + type.name();
  }
}

mixin Mappable {
  T getValue<T>(String fieldName);
}
